/*******************************************************************************

MihPhoto - Photo viewer for multi-touch devices
Copyright (C) 2010 Mihai Paslariu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <QtGui>
#include <stdio.h>

#include "Config.h"
#include "ImageArea.h"
#include "ScreenSettings.h"

/*******************************************************************************
* CONSTRUCTOR / DESTRUCTOR
*******************************************************************************/

ImageArea::ImageArea(QWidget *parent)
    : QWidget(parent)
{
	setAttribute(Qt::WA_StaticContents);
	_dir_view = false;
	_hide_cursor_timer = 0;
	
	_image_viewer = new ScreenViewer();
	_connectSignals( _image_viewer );
	if ( _image_viewer ) _image_viewer->setSize( width(), height() );
	
	_dir_viewer = new ScreenDirectory();
	_connectSignals( _dir_viewer );
	if ( _dir_viewer ) _dir_viewer->setSize( width(), height() );
			
	_front_viewer = NULL;

	connect( &_timer, SIGNAL(timeout()), this, SLOT(onTimer()));
	_timer.start( g_config.timer_duration );
	if ( g_config.multitouch )
	{
		setAttribute(Qt::WA_AcceptTouchEvents);
		setMouseTracking(true);
	} else {
		setMouseTracking(true);
	}
}

ImageArea::~ImageArea()
{
	delete _image_viewer;
	delete _dir_viewer;
	if ( _front_viewer )
		delete _front_viewer;
}

/*******************************************************************************
* PUBLIC METHODS AND SLOTS
*******************************************************************************/

void ImageArea::passCloseOnTouch( void )
{
	emit closeOnTouch();
}

void ImageArea::passChangeFullScreen( void )
{
	emit changeFullscreen();
}

void ImageArea::passLoadFile( void )
{
	emit loadFile();
}

void ImageArea::passLoadDir( void )
{
	emit loadDir();
}

void ImageArea::passConfig( void )
{
	//emit config();
	if ( _front_viewer ) return;
	_front_viewer = new ScreenSettings();
	_connectSignals( _front_viewer );
	if ( _front_viewer ) _front_viewer->setSize( width(), height() );
	update();
}

void ImageArea::setFiles( QString dir_name, QString current )
{
	if ( current == "" && !_dir_view )
		_dir_view = true;
	ScreenBase * w = getCurrentViewer();
	if ( w ) w->loadFiles( dir_name, current );
}

void ImageArea::reloadFiles( void )
{
	if ( _dir_view && _dir_viewer )
		_dir_viewer->reloadFiles();
}

void ImageArea::updateSize( int w, int h )
{
	if ( _dir_viewer ) _dir_viewer->setSize( w,h );
	if ( _image_viewer ) _image_viewer->setSize( w,h );
	if ( _front_viewer ) _front_viewer->setSize( w,h );

	ScreenBase * v = getCurrentViewer();
	if ( v ) v->onResize();
}

/*******************************************************************************
* EVENTS
*******************************************************************************/

void ImageArea::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);
	const QRect rect = event->rect();
	painter.fillRect( rect, g_config.background_color );
	
	ScreenBase * w = getCurrentViewer();
	if ( w ) w->onPaint( painter );
}

void ImageArea::resizeEvent(QResizeEvent *event)
{
	updateSize( width(), height() );
	QWidget::resizeEvent(event);
}

bool ImageArea::event( QEvent * event )
{
    if ( event && event->type() == QEvent::MouseMove )
    {
        emit showCursor();
        _hide_cursor_timer = 5 * 1000;
    }

	ScreenBase * w = getCurrentViewer();
	bool ret = ( w != NULL ) && w->onEvent( event );
	if ( !ret )
        return QWidget::event(event);
	return true;
}

void ImageArea::onKeyPress( QKeyEvent * event )
{
	ScreenBase * w = getCurrentViewer();
	if ( w ) w->onKeyPress( event );
}

void ImageArea::onWheel( QWheelEvent * event )
{
	ScreenBase * w = getCurrentViewer();
	if ( w ) w->onWheel( event );
}

void ImageArea::onTimer( void )
{
	ScreenBase * w = getCurrentViewer();
	if ( w ) w->onTimer();
	if ( _hide_cursor_timer > 0 )
	{
		_hide_cursor_timer -= g_config.timer_duration;
		if ( _hide_cursor_timer <= 0 )
		{
			_hide_cursor_timer = 0;
			emit hideCursor();
		}
	}
}

void ImageArea::onStartTimer( void )
{
	_timer.start( g_config.timer_duration );
}

void ImageArea::onChangeMode( void )
{
	if ( _front_viewer )
	{
		delete _front_viewer;
		_front_viewer = NULL;
		update();
		return;
	}

	// verification
	if ( _image_viewer == NULL || _dir_viewer == NULL ) return;
	
	if ( _dir_view )
	{
		_image_viewer->changeFromOtherViewer( _dir_viewer );
	} else {
		_dir_viewer->changeFromOtherViewer( _image_viewer );
		_dir_viewer->scrollToCurrent();
	}
	_dir_view = !_dir_view;
	update();
}

/*******************************************************************************
* PRIVATE METHODS
*******************************************************************************/

void ImageArea::_connectSignals( ScreenBase * w )
{
	if ( w == NULL ) return;
	connect( w, SIGNAL(updateSignal()), this, SLOT(update()) );
	connect( w, SIGNAL(closeOnTouch()), this, SLOT(passCloseOnTouch()) );
	connect( w, SIGNAL(changeFullscreen()), this, SLOT(passChangeFullScreen()) );
	connect( w, SIGNAL(loadFile()), this, SLOT(passLoadFile()) );
	connect( w, SIGNAL(loadDir()), this, SLOT(passLoadDir()) );
	connect( w, SIGNAL(config()), this, SLOT(passConfig()) );
	connect( w, SIGNAL(startTimer()), this, SLOT(onStartTimer()) );
	connect( w, SIGNAL(changeViewer()), this, SLOT(onChangeMode()) );
}
